/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef ASYNCHRONOUS_SIGNALS_TYPE_SLOT_HPP
#define ASYNCHRONOUS_SIGNALS_TYPE_SLOT_HPP 1

#include <functional>
#include <vector>
#include <asynchronous/signals/slot_iterator.hpp>

namespace asynchronous
{

  namespace signals
  {

    namespace type
    {

      template < typename Signature >
      struct slot
      {
	using slot_type = std::function<Signature>;
	using container_type = std::vector<slot_type>;
	using slot_iterator = slot_iterator_t<typename container_type::iterator>;
	using const_slot_iterator = slot_iterator_t<typename container_type::const_iterator>;
	using type = slot_type;
      };

      template < typename Signature >
      using slot_t = typename slot<Signature>::type;

      template < typename Signature >
      using slot_container_t = typename slot<Signature>::container_type;

      template < typename Signature >
      using slot_iterator_t = typename slot<Signature>::slot_iterator;

      template < typename Signature >
      using const_slot_iterator_t = typename slot<Signature>::const_slot_iterator;

    } // namespace type

  } // namespace signals

} // namespace asynchronous

#endif
