/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_DATAFRAME_REPORTING_POLICY_HPP
#define DATAFRAME_DATAFRAME_REPORTING_POLICY_HPP 1

#include <iostream>
#include <string>
#include <boost/lexical_cast.hpp>
#include <dataframe/dataframe_inserting_policy.hpp>

namespace dataframe
{

  template < typename Policy = dataframe_inserting_policy >
  struct dataframe_reporting_policy
  {
    template < typename Headers, typename Data, typename Line, typename LineNumber >
    static void missing_fields (Headers& headers, Data& data, Line line, LineNumber line_number)
    {
      std::cout
	<< "dataframe(" << fields(headers) << "): "
	<< "line " << line_number << " is missing fields "
	<< "(" << line.size() << "<" << headers.size() << "): "
	<< fields(line,'|') << std::endl;
      Policy::missing_fields(headers,data,line,line_number);
    }
    
    template < typename Headers, typename Data, typename Line, typename LineNumber >
    static void unexpected_fields (Headers& headers, Data& data, Line line, LineNumber line_number)
    {
      std::cout
	<< "dataframe(" << fields(headers) << "): "
	<< "line " << line_number << " has unexpected fields "
	<< "(" << line.size() << ">" << headers.size() << "): "
	<< fields(line,'|') << std::endl;
      Policy::unexpected_fields(headers,data,line,line_number);
    }

  private:
    template < typename Fields >
    static std::string fields (Fields fields, char delimiter = ',')
    {
      std::string result;
      bool first = true;
      for (auto field : fields)
	{
	  if (!first)
	    result += delimiter;
	  result += boost::lexical_cast<std::string>(field);
	  first = false;
	}
      return result;
    }
  };

} // namespace dataframe

#endif
