/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_GRAPH_COLOR_MAP_HPP
#define GRAPH_MODEL_GRAPH_COLOR_MAP_HPP 1

#include <boost/property_map/property_map.hpp>
#include <observer_ptr.hpp>

namespace graph_model
{

  namespace graph
  {

    template < typename ColorMap >
    class color_map
    {
      using pointer = observer_ptr<ColorMap>;
      using vertex_descriptor = typename ColorMap::key_type;
      using color_type = typename ColorMap::mapped_type;

    public:
      using category = boost::read_write_property_map_tag;
      using key_type = vertex_descriptor;
      using value_type = color_type;
      using reference = color_type;

    public:
      color_map (ColorMap& color_map) : p_(make_observer(color_map)) {}

      friend auto get (color_map map, vertex_descriptor v)
      {
	auto i = map.p_->find(v);
	return i == map.p_->end() ? boost::color_traits<color_type>::white() : i->second;
      }

      friend void put (color_map map, vertex_descriptor v, color_type color)
      { (*map.p_)[v] = color; }

    private:
      pointer p_;
    };

    template < typename ColorMap >
    auto make_color_map (ColorMap& cm) { return color_map<ColorMap>(cm); }

  } // namespace graph

} // namespace graph_model

#endif
