/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_GRAPH_VERTEX_INSERTER_HPP
#define GRAPH_MODEL_GRAPH_VERTEX_INSERTER_HPP 1

#include <observer_ptr.hpp>

namespace graph_model
{

  namespace graph
  {

    template < typename Graph >
    class vertex_inserter
    {
      using pointer = observer_ptr<Graph>;

    public:
      vertex_inserter (Graph& graph)
	: p_(make_observer(graph))
      {}

      template < typename VertexDescriptor, typename G >
      void operator () (VertexDescriptor v, G const&) const
      { p_->insert_vertex_descriptor(v); }

      private:
	pointer p_;
      };

    template <>
    struct vertex_inserter< void >
    {
      template < typename VertexDescriptor, typename G >
      void operator () (VertexDescriptor, G const&) const {}
      };

    auto make_vertex_inserter () { return vertex_inserter<void>(); }

    template < typename Graph >
    auto make_vertex_inserter (Graph& graph)
    { return vertex_inserter<Graph>(graph); }

  } // namespace graph
  
} // namespace graph_model

#endif
