/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_RANDOM_DISTRIBUTION_HPP
#define GRAPH_MODEL_RANDOM_DISTRIBUTION_HPP 1

namespace graph_model
{

  namespace random
  {

    template < typename Distribution >
    struct distribution
    {
      static auto& apply () { return distribution_(); }

      template < typename Engine, typename ... Parameters >
      static auto apply (Engine& engine, Parameters&& ... parameters)
      {
	using param_type = typename Distribution::param_type;
	return distribution_()(engine,param_type(std::forward<Parameters>(parameters)...));
      }

      static void reset () { distribution_().reset(); }

    private:
      static Distribution& distribution_ ()
      {
	static Distribution distribution;
	return distribution;
      }
    };

  } // namespace random

} // namespace graph_model

#endif
