/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_SIGNALS_TYPE_INVOCATION_RESULT_HPP
#define GRAPH_MODEL_SIGNALS_TYPE_INVOCATION_RESULT_HPP 1

#include <tuple>
#include <type_traits>
#include <asynchronous/signals/invoke.hpp>

namespace graph_model
{

  namespace signals
  {

    namespace type
    {

      /// \brief Deduce the return type of invoking a signal
      /// \tparam Combiner The type of signal combiner
      /// \tparam SlotIterator The type of slot iterator
      /// \tparam Args... The type of function arguments
      ///
      template < typename Combiner, typename SlotIterator, typename ... Args >
      struct invocation_result
      {
	using type = Combiner;
      };

      template < typename SlotIterator, typename ... Args>
      class invocation_result< void,SlotIterator,Args... >
      {
	using invoke = asynchronous::signals::invoke;
	using tuple_type = std::tuple<Args...>;
	using result_type = decltype(invoke::apply(*std::declval<SlotIterator>(),std::declval<tuple_type>()));

      public:
	using type = result_type;
      };

      template < typename Combiner, typename SlotIterator, typename ... Args >
      using invocation_result_t = typename invocation_result<Combiner,SlotIterator,Args...>::type;

    } // namecpace type

  } // namespace signals

} // namespace graph_model

#endif
