/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_MARKOV_BLANKET_FACTOR_BASE_HPP
#define GRAPH_MODEL_VERTEX_MARKOV_BLANKET_FACTOR_BASE_HPP 1

#include <tuple>

namespace graph_model
{

  namespace vertex
  {

    template < typename Derived >
    class markov_blanket_factor_base
    {
      template < std::size_t I, std::size_t N >
      struct add_factor_helper
      {
	template < typename Factor, typename Tuple >
	static void apply (Factor& factor, Tuple& tuple)
	{
	  std::get<I>(tuple).add_factor(factor);
	  add_factor_helper<I+1,N>::apply(factor,tuple);
	}
      };

      template < std::size_t N >
      struct add_factor_helper<N,N>
      {
	template < typename Factor, typename Tuple >
	static void apply (Factor&, Tuple&) {}
      };

    public:
      template < typename ... T >
      auto& markov_blanket (T& ... t)
      {
	apply(std::forward_as_tuple(t...));
	return derived();
      }

    private:
      template < typename Tuple >
      void apply (Tuple tuple)
      { add_factor_helper<0,std::tuple_size<Tuple>::value>::apply(derived(),tuple); }

    private:
      auto& derived () { return static_cast<Derived&>(*this); }
      auto& derived () const { return static_cast<Derived const&>(*this); }
    };

  } // namespace vertex

} // namespace graph_model

#endif
