/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_OBSERVER_COLLECTION_HPP
#define GRAPH_MODEL_VERTEX_OBSERVER_COLLECTION_HPP 1

#include <functional>
#include <set>
#include <type_traits>
#include <observer_ptr.hpp>
#include <type_traits/is_rvalue.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename T >
    class observer_collection
    {
      using observer_type = observer_ptr<T>;
      using observer_collection_type = std::set<observer_type>;

    public:
      using const_iterator = typename observer_collection_type::const_iterator;

    public:
      auto empty () const { return observers_.empty(); }
      auto size () const { return observers_.size(); }

      auto begin () const { return observers_.begin(); }
      auto end () const { return observers_.end(); }

      template < typename U
		 , typename = std::enable_if_t<std::is_convertible<U*,T*>::value>
		 >
      auto insert (observer_ptr<U> observer)
      {
	observers_.insert(observer);
	return observer;
      }

      template < typename U
		 , typename = std::enable_if_t<std::is_convertible<U*,T*>::value>
		 >
      bool remove (observer_ptr<U> observer)
      {
	return observers_.empty() || (observers_.erase(observer) == 1);
      }

    private:
      observer_collection_type observers_;
    };

  } // namespace vertex

} // namespace graph_model

#endif
