/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef IDENTIFIERS_IDENTIFIER_TAG_HPP
#define IDENTIFIERS_IDENTIFIER_TAG_HPP 1

#include <iosfwd>
#include <string>
#include <boost/lexical_cast.hpp>
#include <boost/operators.hpp>
#include <mpl/enable_constructor.hpp>

namespace identifiers
{

  class identifier_tag
    : boost::totally_ordered<identifier_tag>
  {
    using self = identifier_tag;

  public:
    identifier_tag () = default;

    template < typename T
	       , typename = mpl::enable_constructor_t<self,T>
	       >
    explicit identifier_tag (T&& t)
      : tag_(initialize(std::forward<T>(t)))
    {}

    operator std::string () const { return tag_; }

    friend bool operator == (self const& lhs, self const& rhs)
    { return lhs.tag_ == rhs.tag_; }

    friend bool operator < (self const& lhs, self const& rhs)
    { return lhs.tag_ < rhs.tag_; }

  private:
    template < typename T >
    static auto initialize (T&& t)
    { return boost::lexical_cast<std::string>(std::forward<T>(t)); }

  private:
    std::string tag_;
  };

  std::ostream& operator << (std::ostream& os, identifier_tag tag)
  { return os << static_cast<std::string>(tag); }

  template < typename T >
  auto make_identifier_tag (T&& t) { return identifier_tag(std::forward<T>(t)); }

} // namespace identifiers

#endif
