/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_VECTOR_VECTOR_HPP
#define SAMPLE_SPACE_VECTOR_VECTOR_HPP 1

#include <sample_space/vector/vector_fwd.hpp>
#include <boost/accumulators/accumulators.hpp>
#include <mpl/enable_constructor.hpp>
#include <sample_space/vector/tag/vector.hpp>
#include <sample_space/vector/vector_base.hpp>

namespace sample_space
{

  namespace vector
  {

    /// \brief A vector space class
    /// \tparam T The type of field
    /// \tparam Index The type of index for dimensions
    /// \tparam Tag The tag defining the type of vector space
    /// \tparam Basis The basis for the vector space
    /// \tparam Storage The type of storage for the vector
    ///
    template < typename T
	       , typename Index
	       , typename Tag
	       , typename Basis
	       , typename Storage
	       >
    class vector
      : public vector_base<vector<T,Index,Tag,Basis,Storage>,T,Index,Tag,Basis,Storage>
    {
      using self = vector;
      using base = vector_base<self,T,Index,Tag,Basis,Storage>;

    public:
      template < typename ... Args
		 , typename = ::mpl::enable_constructor_t<self,Args...>
		 >
      vector (Args&& ... args)
	: base(std::forward<Args>(args)...)
      {}

      vector (std::initializer_list<T> x)
	: base(x)
      {}
    };

    template < typename T, typename Index, typename Tag, typename Basis, typename Storage >
    auto zero (vector<T,Index,Tag,Basis,Storage> v) { return v.zero(); }

    template < typename T, typename Index, typename Tag, typename Basis, typename Storage >
    auto one (vector<T,Index,Tag,Basis,Storage> v) { return v.one(); }

    template < typename T
	       , typename Index
	       , typename Tag
	       , typename Basis
	       , typename Storage
	       >
    auto variate (vector<T,Index,Tag,Basis,Storage> v) { return v; }

    template < typename T
	       , typename Index
	       , typename Tag
	       , typename Basis
	       , typename Storage
	       , typename Context
	       >
    auto variate (vector<T,Index,Tag,Basis,Storage> v, Context) { return v; }

  } // namespace vector
  
} // namespace sample_space

namespace boost
{

  namespace numeric
  {

    namespace functional
    {

      template < typename T
		 , typename Index
		 , typename Tag
		 , typename Basis
		 , typename Storage
		 >
      struct tag< sample_space::vector::vector<T,Index,Tag,Basis,Storage> >
      {
	using type = sample_space::vector::tag::vector;
      };

      template< typename Left, typename Right >
      class fdiv< Left,Right,sample_space::vector::tag::vector,void >
      {
	using value_type = typename fdiv<traits::value_t<Left>,Right>::result_type;

      public:
	using result_type = traits::convert_t<Left,value_type>;

        result_type operator () (Left& left, Right& right) const
        { return left / right; }
      };

    } // namespace functional

  } // namespace numeric

} // namespace boost

#endif
