/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef TRAITS_ARRAY_HPP
#define TRAITS_ARRAY_HPP 1

#include <array>
#include <traits/convert.hpp>
#include <traits/domain.hpp>
#include <traits/probability.hpp>
#include <traits/real.hpp>
#include <traits/value.hpp>
#include <traits/variate.hpp>
#include <type_traits/common_type.hpp>

namespace traits
{

  namespace defaults
  {

    template < typename T, std::size_t N, typename U >
    struct convert< std::array<T,N>,U >
    {
      using type = std::array<U,N>;
    };

    template < typename T, std::size_t N, typename U >
    struct convert< std::array<T,N> const,U >
    {
      using type = std::array<U,N> const;
    };

    template < typename T, std::size_t N >
    struct domain< std::array<T,N>,std::false_type >
      : domain<T>
    {};

    template < typename T, std::size_t N >
    struct probability< std::array<T,N> >
      : probability<T>
    {};

    template < typename T, std::size_t N >
    struct real< std::array<T,N>,std::false_type,std::false_type >
      : real<T>
    {};

    template < typename T, std::size_t N >
    struct value< std::array<T,N> >
    {
      using type = T;
    };

    template < typename T, std::size_t N >
    struct variate< std::array<T,N> >
    {
      using type = std::array<T,N>;
    };

  } // namespace defaults

} // namespace traits

namespace type_traits
{

  namespace defaults
  {

    template < typename T, typename U, std::size_t N >
    struct common_type< std::array<T,N>,std::array<U,N> >
    {
      using type = std::array<common_type_t<T,U>,N>;
    };

  } // namespace defaults

} // namespace type_traits

#endif
