/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_distribution_multinomial_multinomial_strategy

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/distribution/multinomial/multinomial_strategy.hpp>
#include <sample_space/vector/simplex.hpp>
#include <sample_space/vector/vector.hpp>

class parameter_pack {};

BOOST_AUTO_TEST_CASE(test_default_constructor)
{
  using multinomial_strategy_type
    = graph_model::distribution::multinomial::multinomial_strategy<parameter_pack>;
  multinomial_strategy_type strategy;
}

BOOST_AUTO_TEST_CASE(test_constructor_function)
{
  using simplex_type = sample_space::vector::simplex<double>;
  using vector_type = sample_space::vector::vector<double>;

  using graph_model::distribution::multinomial::make_multinomial_strategy;
  using graph_model::distribution::parameter::make_parameter;
  using graph_model::distribution::parameter::make_parameter_pack;

  namespace multinomial = graph_model::distribution::multinomial;
  namespace tag = graph_model::distribution::parameter::tag;

  simplex_type variate (3);
  variate.at(2) = 0.1;
  variate.at(1) = 0.2;
  variate.at(0) = 0.7;

  vector_type p (3);
  p.at(2) = 1;
  p.at(1) = 1;
  p.at(0) = 1;

  auto strategy = make_multinomial_strategy
    (make_parameter_pack
     (make_parameter<tag::probability>().default_value(p),
      make_parameter<tag::variate>().default_value(variate)));
  strategy.sample();
  strategy.pdf();
}
