/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE traits_convert

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <traits/convert.hpp>
#include <traits/vector.hpp>
#include <boost/mpl/pair.hpp>
#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector<
  boost::mpl::pair<int,int>
  , boost::mpl::pair<int const,int>
  , boost::mpl::pair<int&,int>
  , boost::mpl::pair<int const&,int>
  , boost::mpl::pair<int&&,int>
  , boost::mpl::pair<int const&&,int>
  >;

using vector_test_types = boost::mpl::vector<
  boost::mpl::pair<int,int>
  , boost::mpl::pair<int const,int>
  >;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_convert,Pair,test_types)
{
  using T1 = typename Pair::first;
  using T2 = typename Pair::second;

  using T = traits::convert_t<T1,T2>;
  static_assert(std::is_same<T,T2>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_convert_vector,Pair,vector_test_types)
{
  using T1 = typename Pair::first;
  using T2 = typename Pair::second;

  using V1 = std::vector<T1>;
  using V2 = std::vector<T2>;

  using T = traits::convert_t<V1,T2>;
  static_assert(std::is_same<T,V2>::value,"");
}
